/*-----------------------------------------------------------------*/
/*                         TLP224 INTERFACE JAVA                   */
/*                                                                 */
/*                         Version 1.0,                            */
/*                                                                 */
/* Creation : 14 Fevrier 1991                                      */
/* Revision :  5 Avril   1998                                      */
/* Auteur    : G. Grimonprez                                       */
/*                                                                 */
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/*                            INCLUDES SYSTEME                     */
/*-----------------------------------------------------------------*/

#include <iostream.h>
#include <conio.h>
#include <setjmp.h>
#include <ctype.h>
#include <process.h>
#include "rs232cc.h"
#include <stdio.h>
#include <windows.h>

#include <gemplus.h>
#include <wscump2.h>

/*-----------------------------------------------------------------*/
/*                            DEFINES                              */
/*-----------------------------------------------------------------*/

#define WORD    unsigned int
#define BYTE    unsigned char

#define MAX_BLOC    256

/*                        PROTOCOLE TLP224                         */

#define ETX                     3

#define ACK_LECTEUR             0x60
#define NAK_LECTEUR             0xe0

#define SELECTION_MODE_COUPLEUR 0x02
#define COUPLEUR_ASYNCHRONE     0x02
#define COUPLEUR_SYNCHRONE      0x03

#define RESET_CARTE             0x6e
#define MISE_HORS_TENSION_CARTE 0x4d
#define ORDRE_ENTRANT_LECTEUR   0xda
#define ORDRE_SORTANT_LECTEUR   0xdb

/*                            ORDRES ISO  Cartes COS               */
#define G_WINDOWS
#define ORDRE_ISO_INCORRECT     0xff
#define ORDRE_ISO_STATUS        0xf2

/*-----------------------------------------------------------------*/
/*                            DECLARATIONS GLOBALES                */
/*-----------------------------------------------------------------*/

jmp_buf jmp_buffer;  /* point de retour pour recuperation d'erreur */

#define MAX_ORDRE 8192

BYTE ordre[MAX_ORDRE];

/*-----------------------------------------------------------------*/
/*                            INCLUDES                             */
/*-----------------------------------------------------------------*/

//------ ERREUR ---------------

void erreur(char msg[])
{
 FILE *f;

 f=fopen("cardout","wt");
 fprintf(f,"%cErreur : %s\n",0,msg);
 fclose(f);
 while(remove("sema"))sleep(1);
 longjmp(jmp_buffer,0);
}

/*-----------------------------------------------------------------*/
/*                            FONCTIONS                            */
/*-----------------------------------------------------------------*/

/*------ EMETTRE_QUARTET_NORME_TLP224 -----------------------------*/

void emettre_quartet_norme_tlp224(BYTE c)
{
 if(c>=10) emission_octet(c+55);
 else      emission_octet(c+48);
}

/*------ EMETTRE_BYTE_NORME_TLP224 --------------------------------*/

void emettre_octet_norme_tlp224(BYTE c)
{

 emettre_quartet_norme_tlp224(c>>4);
 emettre_quartet_norme_tlp224(c & 0x0f);
}

/*------ EMETTRE_MESSAGE_NORME_TLP224 -----------------------------*/

void emettre_message_norme_tlp224(WORD longueur_bloc,BYTE bloc[MAX_BLOC])
{
 BYTE lrc;
 WORD i;

 cout << endl << "EMISSION VERS LA CARTE" << endl;
 emettre_octet_norme_tlp224(ACK_LECTEUR);/* ACK pour le lecteur GCI  */
 lrc=ACK_LECTEUR;

 emettre_octet_norme_tlp224(longueur_bloc); /* longueur du bloc      */
 lrc^=longueur_bloc;

 for(i=0;i<longueur_bloc;i++) {          /* information du message   */
     emettre_octet_norme_tlp224(bloc[i]);
     lrc^=bloc[i];
     }

 emettre_octet_norme_tlp224(lrc);       /* lrc                      */
 emission_octet(ETX);
}

/*------ RECEVOIR_QUARTET_NORME_TLP224 ----------------------------*/

BYTE recevoir_quartet_norme_tlp224(void)
{
 BYTE c;

 c=reception_octet();
 if(c>=58) c-=55;
 else      c-=48;
 return c;
}

/*------ RECEVOIR_OCTET_NORME_TLP224 ------------------------------*/

BYTE recevoir_octet_norme_tlp224(void)
{

 return (recevoir_quartet_norme_tlp224()<<4)|recevoir_quartet_norme_tlp224();
}

/*------ RECEVOIR_MESSAGE_NORME_TLP224 ----------------------------*/

void recevoir_message_norme_tlp224(WORD &longueur_bloc,BYTE bloc[MAX_BLOC])
{
 BYTE lrc;
 WORD i;

 lrc=recevoir_octet_norme_tlp224();
 switch(lrc) {
	case ACK_LECTEUR:
			 break;
	case NAK_LECTEUR:
			 lrc=recevoir_octet_norme_tlp224();
			 if(lrc) erreur("octet nul non recu");
			 lrc=recevoir_octet_norme_tlp224();
			 if(lrc!=0xe0) erreur("lrc incorrect");
			 erreur("NAK recu du lecteur");
			 break;
	default         :
			 erreur("accuse de reception <> de ACK ou NAK");
			 break;
	}
 longueur_bloc=recevoir_octet_norme_tlp224();
 lrc^=longueur_bloc;
 for(i=0;i<longueur_bloc;i++) {
     bloc[i]=recevoir_octet_norme_tlp224();
     lrc^=bloc[i];
     }
 if(recevoir_octet_norme_tlp224()!=lrc) erreur("lrc incorrect");
 if(reception_octet()!=ETX) erreur("ETX non recu");

}

//------ EXISTS ----------------------------------------------

int exists(char nom[])
{
 FILE *f;

 f=fopen(nom,"rb");
 if(f==NULL) return 0;
 fclose(f);
 return 1;
}

//------ TRAITER_DEMANDE --------------------------------------

void traiter_demande(void)
{
 FILE *f;
 WORD n;
 BYTE c;

 f=fopen("cardin","rb");
 if(f==NULL) erreur("Fichier cardin non trouve");
 n=fread(ordre,1,MAX_ORDRE,f);
 fclose(f);
 if(n==0) erreur("ordre incorrect");
 emettre_message_norme_tlp224(n,ordre);
 recevoir_message_norme_tlp224(n,ordre);
 f=fopen("cardout","wb");
 c=1;
 fwrite(&c,1,1,f);
 fwrite(ordre,1,n,f);
 fclose(f);
 while(remove("sema"))sleep(1);
}

//------ ATTENDRE ---------------------------------------------

void attendre(void)
{

 do {
     if(exists("sema")) traiter_demande();
     sleep(1);
     putch('.');
    }while(!kbhit());
 getch();
}

/*------ MAIN  -----------------------------------------------------*/

void main(void)
{
 cout<<"debut";
 const WORD16 a1=1;
 BYTE G_FAR* a2;
 INT16 rep;
 rep=G_MPC_InitLib(a1,a2);
 if(exists("sema")) while(remove("sema")) sleep(1);
 init_controleur(INTERRUPT,COM2,9600,8,1,SANS,10);
 setjmp(jmp_buffer);
 attendre();
 restaure_controleur();
 exit(0);
}
